﻿using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Controls.Primitives;
using System.Windows.Input;
using Varuna.Modules.Explorer;

namespace Varuna
{
    public partial class MainWindow : Window
    {
        public ObservableCollection<DriveInfo> Disks { get; set; }
        public IEnumerable<Explorer> Explorers => this.ExplorerManager.Items.OfType<Explorer>();

        public MainWindow()
        {
            this.ResetDriveList();
            InitializeComponent();
        }

        private void ResetDriveList()
        {
            this.Disks = new ObservableCollection<DriveInfo>(DriveInfo.GetDrives().Where(x => x.IsReady));
        }

        public void ChangePath(bool newTab, bool openNewTab, string newPath)
        {
            Explorer expl = (Explorer)this.ExplorerManager.SelectedItem;
            if (newTab || expl == null)
            {
                expl = new Explorer(this, newPath);
                this.ExplorerManager.Items.Add(expl);
                if (openNewTab)
                    this.ExplorerManager.SelectedItem = expl;
            }
            else
                expl.CurrentPath = newPath;
        }

        private void Disk_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            if (((ListBox)sender).SelectedItem != null && Mouse.PrimaryDevice.LeftButton == MouseButtonState.Pressed)
            {
                DriveInfo dinfo = (DriveInfo)((ListBox)sender).SelectedItem;
                this.ChangePath(false, true, dinfo.RootDirectory.FullName);
                this.DiskListBox.SelectedItem = null;
            }
        }

        private void newTab_Click(object sender, RoutedEventArgs e)
        {
            DriveInfo dinfo = (DriveInfo)this.DiskListBox.SelectedItem;
            this.ChangePath(true, true, dinfo.RootDirectory.FullName);
            this.DiskListBox.SelectedItem = null;
        }

        private void TabItem_PreviewMouseMove(object sender, MouseEventArgs e)
        {
            Explorer tabItem = e.Source as Explorer;

            if (tabItem != null && Mouse.PrimaryDevice.LeftButton == MouseButtonState.Pressed)
                DragDrop.DoDragDrop(this.ExplorerManager, tabItem, DragDropEffects.All);
        }

        private void TabItem_Drop(object sender, DragEventArgs e)
        {
            Explorer tabItemSource = e.Data.GetData(typeof(Explorer)) as Explorer;
            if (tabItemSource != null)
            {
                if (e.Source is Explorer)
                {
                    var tabItemTarget = e.Source as Explorer;
                    if (!tabItemTarget.Equals(tabItemSource))
                    {
                        this.ExplorerManager.Items.Remove(tabItemSource);
                        this.ExplorerManager.Items.Insert(this.ExplorerManager.Items.IndexOf(tabItemTarget)+1, tabItemSource);
                        this.ExplorerManager.SelectedItem = tabItemSource;
                    }
                }
                else if (e.OriginalSource is TabPanel)
                {
                    this.ExplorerManager.Items.Remove(tabItemSource);
                    this.ExplorerManager.Items.Add(tabItemSource);
                    this.ExplorerManager.SelectedItem = tabItemSource;
                }
            }
        }

        private void ListBox_ContextMenuOpening(object sender, ContextMenuEventArgs e)
        {
            e.Handled = e.OriginalSource.GetType() == typeof(ScrollViewer);
        }
    }
}
