﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Diagnostics;
using System.IO;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Input;
using Varuna.Modules.Utils.Windows;

namespace Varuna.Modules.Explorer
{
    public partial class Explorer : TabItem, INotifyPropertyChanged
    {
        public MainWindow Window { get; set; }

        private string currentPath;
        public string CurrentPath
        {
            get { return currentPath; }
            set
            {
                currentPath = value;
                this.OnPropertyChanged("CurrentPath");
                this.OnPropertyChanged("PathHistory");
                this.Files = new ObservableCollection<FileInfo>(Directory.GetFileSystemEntries(value).Select(x => new FileInfo(x)));
                this.Search = "";
            }
        }

        private string commonName = "";
        public string CommonName
        {
            get { return commonName; }
            set
            {
                commonName = value;
                this.OnPropertyChanged("CommonName");
            }
        }

        private string search;
        public string Search
        {
            get { return search; }
            set { search = value;
                OnPropertyChanged("Search");
                if (!String.IsNullOrEmpty(value))
                    this.Files = new ObservableCollection<FileInfo>(Directory.GetFileSystemEntries(this.CurrentPath, value).Select(x => new FileInfo(x)));
            }
        }

        private ObservableCollection<FileInfo> files;
        public ObservableCollection<FileInfo> Files
        {
            get { return files; }
            set
            {
                files = value;
                OnPropertyChanged("Files");
            }
        }

        public ObservableCollection<string> PathHistory { get; set; } = new ObservableCollection<string>();

        public Explorer(MainWindow window, string currentPath)
        {
            this.Window = window;
            this.CurrentPath = currentPath;

            InitializeComponent();
        }

        public event PropertyChangedEventHandler PropertyChanged;
        private void OnPropertyChanged(string name)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(name));
        }

        private void Rename_Click(object sender, System.Windows.RoutedEventArgs e)
        {
            AskText ask = new AskText("Titre pour l'onglet", "Ecrivez le nouveau nom pour l'onglet", this.CommonName);
            this.CommonName = ask.Result;
        }

        private void Previous_Click(object sender, System.Windows.RoutedEventArgs e)
        {
            string pathToGo = this.PathHistory.Last();
            this.PathHistory.RemoveAt(this.PathHistory.Count - 1);
            this.CurrentPath = pathToGo;
        }

        private void Parent_Click(object sender, System.Windows.RoutedEventArgs e)
        {
            var dir = Directory.GetParent(this.CurrentPath);
            if ((dir.Attributes & FileAttributes.Directory) != 0)
            {
                if (dir.CheckAccessGranted())
                { 
                    this.PathHistory.Add(this.CurrentPath);
                    this.CurrentPath = dir.FullName;
                }
                else
                    System.Windows.MessageBox.Show("Je n'ai pas l'autorisation d'ouvrir ce dossier. :(", "Autorisation insuffisante", System.Windows.MessageBoxButton.OK, System.Windows.MessageBoxImage.Exclamation);
            }
        }

        private void StackPanel_MouseLeftButtonUp(object sender, System.Windows.Input.MouseButtonEventArgs e)
        {
            FileInfo file = (FileInfo)((StackPanel)sender).Tag;
            if ((file.Attributes & FileAttributes.Directory) != 0)
            {
                if (file.CheckAccessGranted())
                {
                    this.PathHistory.Add(this.CurrentPath);
                    this.CurrentPath = file.FullName;
                }
                else
                    System.Windows.MessageBox.Show("Je n'ai pas l'autorisation d'ouvrir ce dossier. :(", "Autorisation insuffisante", System.Windows.MessageBoxButton.OK, System.Windows.MessageBoxImage.Exclamation);

            }
            else if (!String.IsNullOrEmpty(ProcessHelper.FindExecutable(file.FullName)))
                Process.Start(ProcessHelper.FindExecutable(file.FullName), file.FullName);
        }

        private void ListView_ContextMenuOpening(object sender, ContextMenuEventArgs e)
        {
            e.Handled = e.OriginalSource.GetType() == typeof(ScrollViewer);
        }

        private void TextBox_KeyDown(object sender, System.Windows.Input.KeyEventArgs e)
        {
            if (e.Key == System.Windows.Input.Key.Enter)
                BindingOperations.GetBindingExpression((TextBox)sender, TextBox.TextProperty).UpdateSource();
            else if (e.Key == System.Windows.Input.Key.Escape)
                this.CurrentPath = this.CurrentPath;
        }

        private void PathTextBox_KeyDown(object sender, System.Windows.Input.KeyEventArgs e)
        {
            if (e.Key == System.Windows.Input.Key.Enter)
            {
                bool _pathGoBack = false;

                string text = ((TextBox)sender).Text;
                if (Directory.Exists(text))
                {
                    FileInfo fi = new FileInfo(text);
                    if ((fi.Attributes & FileAttributes.Directory) != 0)
                    {
                        if (fi.CheckAccessGranted())
                        {
                            this.PathHistory.Add(this.CurrentPath);
                            this.CurrentPath = fi.FullName;
                        }
                        else
                        {
                            System.Windows.MessageBox.Show("Je n'ai pas l'autorisation d'ouvrir ce dossier. :(", "Autorisation insuffisante", System.Windows.MessageBoxButton.OK, System.Windows.MessageBoxImage.Exclamation);
                            _pathGoBack = true;
                        }
                    }
                }
                else
                {
                    System.Windows.MessageBox.Show("Ce dossier n'existe pas !", "Problème de chemin", System.Windows.MessageBoxButton.OK, System.Windows.MessageBoxImage.Exclamation);
                    _pathGoBack = true;
                }

                if (_pathGoBack)
                    BindingOperations.GetBindingExpression((TextBox)sender, TextBox.TextProperty).UpdateTarget();

                ((TextBox)sender).SelectionLength = 0;
                ((TextBox)sender).CaretIndex = this.CurrentPath.Length;
            }
        }

        private void Close_Click(object sender, System.Windows.RoutedEventArgs e)
        {
            ((TabControl)this.Parent).Items.Remove(this);
        }

        private void newTab_Click(object sender, RoutedEventArgs e)
        {
            List<string> _cantopen = new List<string>();
            foreach (FileInfo file in ListViewFiles.SelectedItems)
            {
                if (file.CheckAccessGranted())
                    this.Window.ChangePath(true, ListViewFiles.SelectedItems.Count == 1, file.FullName);
                else
                    _cantopen.Add(file.Name);
            }

            if (_cantopen.Count > 0)
                _cantopen.ForEach(x => { MessageBox.Show($"Je ne peux pas ouvrir le dossier \"{x}\" en raison de son niveau de protection. :(", "Problème d'ouverture", MessageBoxButton.OK, MessageBoxImage.Information); });
        }

        private void TabItem_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.Key == Key.W && (Keyboard.IsKeyDown(Key.LeftCtrl) || Keyboard.IsKeyDown(Key.RightCtrl)))
                this.Close_Click(null, null);
            else if (e.Key == Key.V && (Keyboard.IsKeyDown(Key.LeftCtrl) || Keyboard.IsKeyDown(Key.RightCtrl)))
            {
                if (Clipboard.GetFileDropList().Count > 0)
                    foreach (string str in Clipboard.GetFileDropList())
                    {
                        string destination = Path.Combine(this.CurrentPath, (new FileInfo(str)).Name);
                        if (!File.Exists(destination) || MessageBox.Show($"La destination existe déjà ({(new FileInfo(str)).Name})\nVoulez-vous écraser cette destination ?", "Destination déjà existante", MessageBoxButton.YesNo, MessageBoxImage.Question) == MessageBoxResult.Yes)
                        {
                            File.Copy(str, destination, true);
                            this.Files.Add(new FileInfo(destination));
                        }
                    }
            }
        }
    }
}
